package net.wanji.utc.controller;

import net.wanji.utc.common.Result;
import net.wanji.utc.common.exception.ControlException;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.service.ControlCommandService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

import static net.wanji.utc.common.ResultEnum.BODY_NOT_MATCH;

/**
 * 控制指令接口
 *
 * @author wuxiaokai
 * @date 2022/11/15 13:21:10
 */
@RestController
@RequestMapping("/controlCommand")
public class ControlCommandController {

    @Value("${signal.mock}")
    private boolean mock;

    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Resource
    private ControlCommandService hkControlCommandService;

    /**
     * 全红控制
     *
     * @param signalId 信号机id
     * @param command  指令
     * @return {@link Result}<{@link T}>
     */
    @PostMapping("/allRedControl")
    public <T> Result<T> allRedControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) return Result.success();
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return hkControlCommandService.allRedControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 黄闪控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @return {@link Result}<{@link T}>
     */
    @PostMapping("/yellowLightControl")
    public <T> Result<T> yellowLightControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) return Result.success();
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return hkControlCommandService.yellowLightControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 关灯控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @return {@link Result}<{@link T}>
     */
    @PostMapping("/closeLightControl")
    public <T> Result<T> closeLightControl(@RequestParam String signalId, @RequestParam Integer command) {
        if (mock) return Result.success();
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return hkControlCommandService.closeLightControl(signalId, null, null, command);
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 步进控制
     *
     * @param signalId 信号id
     * @param command  命令
     * @param stepNum  一步num
     * @return {@link Result}<{@link T}>
     */
    @PostMapping("/stepControl")
    public <T> Result<T> stepControl(@RequestParam String signalId,
                                     @RequestParam Integer command,
                                     @RequestParam int stepNum) {
        if (mock) return Result.success();
        Integer manufacturerId = check(signalId);
        if (manufacturerId.equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            return hkControlCommandService.stepControl(signalId, null, null, command, stepNum);
        } else {
            // todo else
            return null;
        }
    }

    private Integer check(String signalId) {
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByCode(signalId);
        if (crossInfoPO == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，信号机ID不正确。");
        }
        return crossInfoPO.getManufacturerId();
    }
}
